"""
python-can documentation build configuration file

This file is execfile()d with the current directory set to its containing dir.
"""

# -- Imports -------------------------------------------------------------------

import ctypes
import os
import sys
from importlib.metadata import version as get_version
from unittest.mock import MagicMock

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath(".."))

from can import ctypesutil  # pylint: disable=wrong-import-position

# -- General configuration -----------------------------------------------------

# pylint: disable=invalid-name

# The version info for the project, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The full version, including alpha/beta/rc tags.
release: str = get_version("python-can")
# The short X.Y version.
version = ".".join(release.split(".")[:2])

# General information about the project.
project = "python-can"

primary_domain = "py"

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.extlinks",
    "sphinx.ext.todo",
    "sphinx.ext.intersphinx",
    "sphinx.ext.coverage",
    "sphinx.ext.viewcode",
    "sphinx.ext.graphviz",
    "sphinxcontrib.programoutput",
    "sphinx_inline_tabs",
    "sphinx_copybutton",
]

# Now, you can use the alias name as a new role, e.g. :issue:`123`.
extlinks = {"issue": ("https://github.com/hardbyte/python-can/issues/%s/", "issue #%s")}

intersphinx_mapping = {"python": ("https://docs.python.org/3/", None)}

# If this is True, todo and todolist produce output, else they produce nothing.
# The default is False.
todo_include_todos = True

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

graphviz_output_format = "png"  # 'svg'

# The suffix of source filenames.
source_suffix = {".rst": "restructuredtext"}

# The encoding of source files.
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = "index"

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
language = "en"

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
# today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ["_build"]

# The reST default role (used for this markup: `text`) to use for all documents
# default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
# add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# Include documentation from both the class level and __init__
autoclass_content = "both"

# The default autodoc directive flags
autodoc_default_flags = ["members", "show-inheritance"]

# Keep cached intersphinx inventories indefinitely
intersphinx_cache_limit = -1

# location of typehints
autodoc_typehints = "description"

# disable specific warnings
nitpick_ignore = [
    # Ignore warnings for type aliases. Remove once Sphinx supports PEP613
    ("py:class", "OpenTextModeUpdating"),
    ("py:class", "OpenTextModeWriting"),
    ("py:class", "OpenBinaryModeUpdating"),
    ("py:class", "OpenBinaryModeWriting"),
    ("py:class", "OpenTextModeReading"),
    ("py:class", "OpenBinaryModeReading"),
    ("py:class", "BusConfig"),
    ("py:class", "can.typechecking.BusConfig"),
    ("py:class", "can.typechecking.CanFilter"),
    ("py:class", "can.typechecking.CanFilterExtended"),
    ("py:class", "can.typechecking.AutoDetectedConfig"),
    ("py:class", "can.util.T1"),
    ("py:class", "can.util.T2"),
    ("py:class", "~P1"),
    # intersphinx fails to reference some builtins
    ("py:class", "asyncio.events.AbstractEventLoop"),
    ("py:class", "_thread.lock"),
]

# mock windows specific attributes
autodoc_mock_imports = ["win32com", "pythoncom"]
ctypes.windll = MagicMock()
ctypesutil.HRESULT = ctypes.c_long

# -- Options for HTML output --------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = "furo"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
# html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
# html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
# html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
# html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs. This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
# html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ['_static']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
# html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
# html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
# html_additional_pages = {}

# If false, no module index is generated.
# html_domain_indices = True

# If false, no index is generated.
# html_use_index = True

# If true, the index is split into individual pages for each letter.
# html_split_index = False

# If true, links to the reST sources are added to the pages.
# html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
# html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
# html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
# html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = "python-can"
